import { ActionFormData } from '@minecraft/server-ui';
import { typeIdToID, typeIdToDataId } from "./typeId.js";
import { BlockTypes, ItemTypes } from '@minecraft/server';
import { TextureList } from '../textureList.js';

/**
 * Credit:
 * Maintained by Herobrine64 & LeGend077.
*/
const items = ItemTypes.getAll().filter(item => (!item.id.startsWith("minecraft:") && !item.id.endsWith("spawn_egg") && !BlockTypes.get(item.id)))
let number_of_1_16_100_items = items.length;

console.warn(`[System] Custom item amount: ${number_of_1_16_100_items} item(s).`)
const sizes = new Map([
	['inv', [`§i§n§v§t§r§a§d§e§r`, 54]],
	['trade_left', [`§t§r§a§d§e§t§r§a§d§e§1§r`, 54]], ['trade_right', [`§t§r§a§d§e§t§r§a§d§e§2§r`, 54]]
]);

export default class ChestFormData {
	#titleText; #buttonArray;
	constructor(size = 'small') {
		const sizing = sizes.get(size) ?? ['§t§r§a§d§e§2§7§r', 27];
		/** @internal */
		this.#titleText = sizing[0];
		/** @internal */
		this.#buttonArray = [];
		for (let i = 0; i < sizing[1]; i++)
			this.#buttonArray.push(['', undefined]);
		this.slotCount = sizing[1];
	}
	title(text) {
		this.#titleText += text;
		return this;
	}
	button(slot, itemName, itemDesc, texture, stackSize = 1, enchanted = false) {
		const numberCustomItems = number_of_1_16_100_items
		const ID = typeIdToDataId.get(texture) ?? typeIdToID.get(texture);
		this.#buttonArray.splice(slot, 1, [`stack#${Math.min(Math.max(stackSize, 1) || 1, 99).toString().padStart(2, '0')}§r${itemName ?? ''}§r${itemDesc?.length ? `\n§r${itemDesc.join('\n§r')}` : ''}`,
		(((ID + (ID < 262 ? 0 : numberCustomItems)) * 65536) + (!!enchanted * 32768)) || (TextureList[texture] ?? texture)
		]);
		return this;
	}
	pattern(from, pattern, key) {
		for (let i = 0; i < pattern.length; i++) {
			const row = pattern[i];
			for (let j = 0; j < row.length; j++) {
				const letter = row.charAt(j);
				if (key[letter]) {
					const slot = from[1] + j + (from[0] + i) * 9; // Calculate slot index
					const data = key[letter].data;
					const itemName = data.itemName ?? ""
					const itemDesc = data.itemDesc ?? []
					const texture = key[letter].iconPath ?? ""
					const stackSize = data.stackAmount ?? 1
					const enchanted = data.enchanted ?? false
					// console.warn(itemName, itemDesc, texture, stackSize, enchanted)
					this.button(slot, itemName, itemDesc, texture, stackSize, enchanted)
				}
			}
		}
		return this;
	}
	show(player) {
		const form = new ActionFormData()
			.title(this.#titleText);
		this.#buttonArray.forEach(button => {
			form.button(button[0], button[1]?.toString());
		})
		return form.show(player)
	}
}